#import "BidmadsdkFlutterPlugin.h"
#import "FLRSynth.h"

@interface StreamHandlerToFlutter : NSObject<BIDMADBannerDelegate, BIDMADInterstitialDelegate, BIDMADRewardVideoDelegate>

//@property (strong, nonatomic) NSMutableDictionary* listeners;
@property (strong, nonatomic) NSMutableArray * bannerInstances; // Dictionary Containing callback instances of taking out one instance of BIDMADBanner
@property (strong, nonatomic) NSMutableArray * interstitialInstances; // Dictionary Containing callback instances of taking out one instance of BIDMADInterstitial
@property (strong, nonatomic) NSMutableArray * rewardVideoInstances; // Dictionary Containing callback instances of taking out one instance of BIDMADRewardVideo
@property (strong, nonatomic) NSArray* arrayOfArgs;
@property (nonatomic) BOOL isDebug;
@property (strong, nonatomic) NSString* typeOfAd;
@property (strong, nonatomic) NSString* zoneId;
@property (nonatomic) CGRect bannerCGRect;
@property (strong, nonatomic) UIViewController* rootViewController;
@property (strong, nonatomic) NSNumber* refreshInterval;
@property (nonatomic, assign) BOOL isLoaded;

+ (id)sharedInstance;

- (NSString *) bannerInstanceInit: (NSString *)zoneId withPosition: (CGPoint) position; // Returns hash string key for dictionary
- (NSString *) interstitialInstanceInit: (NSString *)zoneId; // Returns hash string key for dictionary
- (NSString *) rewardVideoInstanceInit:  (NSString *)zoneId; // Returns hash string key for dictionary

@end

@implementation BidmadsdkFlutterPlugin

- (instancetype)init {
  self = [super init];
    
  return self;
}

- (void)dealloc {
}

+ (id)sharedInstance{
    static BidmadsdkFlutterPlugin* shared = nil;
    
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        shared = [[BidmadsdkFlutterPlugin alloc] init];
    });
    
    return shared;
}

+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    
    __weak BidmadsdkFlutterPlugin* pluginSelf = [BidmadsdkFlutterPlugin sharedInstance];
    
    [pluginSelf setChannelName: @"com.bidmad.sdk.common/Flutter"];
    [pluginSelf setFlutterChannel:[FlutterMethodChannel
                            methodChannelWithName:pluginSelf.channelName
                            binaryMessenger:[registrar messenger]]];
    
    BidmadsdkFlutterPlugin* instance = [[BidmadsdkFlutterPlugin alloc] init];
    [registrar addMethodCallDelegate:instance channel:pluginSelf.flutterChannel];
    
    [pluginSelf.flutterChannel setMethodCallHandler:^(FlutterMethodCall * call, FlutterResult result) {
        StreamHandlerToFlutter* handler = [StreamHandlerToFlutter sharedInstance];
        
        if ([call.method isEqualToString: @"initBannerChannel"]) {
            result(pluginSelf.channelName);
        } else if ([call.method isEqualToString: @"initInterstitialChannel"]) {
            result(pluginSelf.channelName);
        } else if ([call.method isEqualToString: @"initRewardChannel"]) {
            result(pluginSelf.channelName);
        } else if ([call.method isEqualToString:@"makeAdView"]) {
            [handler setTypeOfAd: @"BANNER"];
            result(nil);
        } else if ([call.method isEqualToString: @"makeInterstitial"]) {
            [handler setTypeOfAd: @"INTERSTITIAL"];
            result(nil);
        } else if ([call.method isEqualToString:@"makeReward"]) {
            [handler setTypeOfAd: @"REWARD"];
            result(nil);
        } else if ([call.method isEqualToString: @"load"]) {
            
            if ([handler.typeOfAd hasPrefix:@"BANNER"]) {
                
                NSNumber* xPoint;
                if ([handler.typeOfAd hasSuffix:@"320X50"]) {
                    xPoint = [NSNumber numberWithFloat: ((UIScreen.mainScreen.bounds.size.width - 320.0)/2)];
                } else if ([handler.typeOfAd hasSuffix:@"300X250"]) {
                    xPoint = [NSNumber numberWithFloat: ((UIScreen.mainScreen.bounds.size.width - 300.0)/2)];
                } else {
                    // DEFAULT CASE WILL BE 320X50
                    xPoint = [NSNumber numberWithFloat: ((UIScreen.mainScreen.bounds.size.width - 320.0)/2)];
                }
                
                CGPoint adPointCentered = CGPointMake( [xPoint floatValue], [((NSNumber*)(((NSDictionary *) call.arguments)[@"y"])) floatValue] );
                
                NSString* instantiationResult = [handler bannerInstanceInit:handler.zoneId withPosition:adPointCentered];
                if ( [instantiationResult hasPrefix:@"SUCCESS"] ) {
                    [((BIDMADBanner*)[handler.bannerInstances lastObject]) requestBannerView];
                    result(nil);
                } else {
                    FlutterError* error = [FlutterError errorWithCode:@"ERROR" message:instantiationResult details:nil];
                    result(error);
                }
                
            } else if ([handler.typeOfAd hasPrefix:@"INTERSTITIAL"]) {
                
                NSString* instantiationResult = [handler interstitialInstanceInit:handler.zoneId];
                if ( [instantiationResult hasPrefix: @"SUCCESS"] ) {
                    NSLog(@"load");
                    [((BIDMADInterstitial*)[handler.interstitialInstances lastObject]) loadInterstitialView];
                    result(nil);
                } else {
                    FlutterError* error = [FlutterError errorWithCode:@"ERROR" message:instantiationResult details:nil];
                    result(error);
                }
                
            } else if ([handler.typeOfAd hasPrefix:@"REWARD"]) {
                
                NSString* instantiationResult = [handler rewardVideoInstanceInit:handler.zoneId];
                if ( [instantiationResult hasPrefix:@"SUCCESS"] ) {
                    [((BIDMADRewardVideo*)[handler.rewardVideoInstances lastObject]) loadRewardVideo];
                    result(nil);
                } else {
                    FlutterError* error = [FlutterError errorWithCode:@"ERROR" message:instantiationResult details:nil];
                    result(error);
                }
                
            }
         
        } else if ([call.method isEqualToString: @"loadXY"]) {
            
            CGPoint adPointCentered = CGPointMake(
                                      [((NSNumber*)(((NSDictionary *) call.arguments)[@"x"])) floatValue],
                                      [((NSNumber*)(((NSDictionary *) call.arguments)[@"y"])) floatValue]
                                      );
            
            NSString* instantiationResult = [handler bannerInstanceInit:handler.zoneId withPosition:adPointCentered];
            if ( [instantiationResult hasPrefix:@"SUCCESS"] ) {
                [((BIDMADBanner*)[handler.bannerInstances lastObject]) requestBannerView];
                result(nil);
            } else {
                FlutterError* error = [FlutterError errorWithCode:@"ERROR" message:instantiationResult details:nil];
                result(error);
            }
            
        } else if ([call.method isEqualToString: @"setInterval"]) {
            
            NSNumber* intervalSec = ((NSNumber*)(((NSDictionary *) call.arguments)[@"sec"]));
            [handler setRefreshInterval: intervalSec];
        
        } else if ([call.method isEqualToString: @"hideBanner"]) {
            
            for ( BIDMADBanner* eachBanner in handler.bannerInstances ) {
                [eachBanner hideView];
            }
        
        } else if ([call.method isEqualToString: @"showBanner"]) {
            
            for ( BIDMADBanner* eachBanner in handler.bannerInstances ) {
                [eachBanner showView];
            }
        
        } else if ([call.method isEqualToString: @"removeBanner"]) {
            
            for ( BIDMADBanner* eachBanner in handler.bannerInstances ) {
                [eachBanner removeAds];
            }
            
        } else if ([call.method isEqualToString: @"show"]) {
            
            if ([handler.typeOfAd hasPrefix:@"INTERSTITIAL"]) {
                
                [((BIDMADInterstitial*)[handler.interstitialInstances lastObject]) showInterstitialView];
                result(nil);
                
            } else if ([handler.typeOfAd hasPrefix:@"REWARD"]) {
                
                [((BIDMADRewardVideo*)[handler.rewardVideoInstances lastObject]) showRewardVideo];
                result(nil);
                
            }
            
        } else if ([call.method isEqualToString: @"isLoaded"]) {
            
            int theEndTime = 1000; // 10 seconds of waiting before calling onFailed
            for (int howManyTimes = 1; howManyTimes <= theEndTime; howManyTimes++) {
                [pluginSelf performSelector:@selector(checkIsLoaded:) withObject:result afterDelay:(0.01 * howManyTimes)];
            }
            [pluginSelf performSelector:@selector(returnFailed:) withObject:result afterDelay:(0.01 * theEndTime)];
            
        } else if ([call.method isEqualToString: @"setAdInfo"]) {
            
            [handler setZoneId:(NSString*)call.arguments];
            NSLog(@"setAdInfo : %@", (NSString*)call.arguments);
            result(nil);
            
        }
            
    }];
    
}

- (void) checkIsLoaded: (FlutterResult) result {
    StreamHandlerToFlutter* handler = StreamHandlerToFlutter.sharedInstance;
    
    if (handler.isLoaded) {
        [NSObject cancelPreviousPerformRequestsWithTarget: self];
        BOOL boolResult = handler.isLoaded;
        [handler setIsLoaded: NO];
        result([NSNumber numberWithBool: boolResult]);
        
//        [NSObject cancelPreviousPerformRequestsWithTarget: self selector:@selector(checkIsLoaded:) object:result];
    }
}

- (void) returnFailed: (FlutterResult) result {
    StreamHandlerToFlutter* handler = StreamHandlerToFlutter.sharedInstance;
    result([NSNumber numberWithBool: handler.isLoaded]);
}

@end

@implementation StreamHandlerToFlutter

+ (id)sharedInstance{
    static StreamHandlerToFlutter* shared = nil;
    
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        shared = [[StreamHandlerToFlutter alloc] init];
    });
    
    return shared;
}

// Returns whether the banner instance initialization was successful
- (NSString *) bannerInstanceInit: (NSString *)zoneId withPosition: (CGPoint) position {
    
    if (self.bannerInstances == nil) {
        [self setBannerInstances: [NSMutableArray new]];
    }
    
    UIViewController * topVC = [[[UIApplication sharedApplication] keyWindow] rootViewController];
    
    if (topVC != nil) {
        while ( topVC.presentedViewController != nil ) {
            topVC = topVC.presentedViewController;
        }
        [[BIDMADSetting sharedInstance] setIsDebug: self.isDebug];
        [self.bannerInstances addObject:[[BIDMADBanner alloc] initWithParentViewController:topVC adsPosition:position bannerSize:banner_320_50]];
        [((BIDMADBanner*)[self.bannerInstances lastObject]) setZoneID:zoneId];
        [((BIDMADBanner*)[self.bannerInstances lastObject]) setDelegate: self];
        [((BIDMADBanner*)[self.bannerInstances lastObject]) setRefreshInterval: self.refreshInterval != nil ? [self.refreshInterval intValue] : 60];
        return @"SUCCESS : Banner Initialization";
    } else {
        // CALL onFailAd
        return @"FAIL : Unable to fetch the top view controller";
    }
    
}

// Returns whether the interstitial instance initialization was successful
- (NSString *) interstitialInstanceInit: (NSString *)zoneId {
    
    if (self.interstitialInstances == nil) {
        [self setInterstitialInstances: [NSMutableArray new]];
    }
    
    NSLog(@"top view controller setting");
    
    UIViewController * topVC = [[[UIApplication sharedApplication] keyWindow] rootViewController];
    
    if (topVC != nil) {
        while ( topVC.presentedViewController != nil ) {
            topVC = topVC.presentedViewController;
            NSLog(@"topVC : %@", topVC.description);
        }
        NSLog(@"Interstitial Settning");
        [[BIDMADSetting sharedInstance] setIsDebug: self.isDebug];
        [self.interstitialInstances addObject: [[BIDMADInterstitial alloc] init]];
        [((BIDMADInterstitial*)[self.interstitialInstances lastObject]) setParentViewController:topVC];
        [((BIDMADInterstitial*)[self.interstitialInstances lastObject]) setZoneID:zoneId];
        [((BIDMADInterstitial*)[self.interstitialInstances lastObject]) setDelegate:self];
        NSLog(@"interstitialInstance set : %@", ((BIDMADInterstitial*)[self.interstitialInstances lastObject]).zoneID);
        NSLog(@"interstitialInstance set SUCCESS");
        return @"SUCCESS : Interstitial Initialization";
    } else {
        // CALL onFailAd
        NSLog(@"interstitialInstance set FAIL");
        return @"FAIL : Unable to fetch the top view controller";
    }
    
}

// Returns whether the reward video instance initialization was successful
- (NSString *) rewardVideoInstanceInit:  (NSString *)zoneId {
    
    if (self.rewardVideoInstances == nil) {
        [self setRewardVideoInstances: [NSMutableArray new]];
    }
    
    UIViewController * topVC = [[[UIApplication sharedApplication] keyWindow] rootViewController];
    
    if (topVC != nil) {
        while ( topVC.presentedViewController != nil ) {
            topVC = topVC.presentedViewController;
        }
        [[BIDMADSetting sharedInstance] setIsDebug: self.isDebug];
        [self.rewardVideoInstances addObject: [[BIDMADRewardVideo alloc] init]];
        [((BIDMADRewardVideo*)[self.rewardVideoInstances lastObject]) setZoneID:zoneId];
        [((BIDMADRewardVideo*)[self.rewardVideoInstances lastObject]) setParentViewController:topVC];
        [((BIDMADRewardVideo*)[self.rewardVideoInstances lastObject]) setDelegate:self];
        return @"SUCCESS : Reward Video Initialization";
    } else {
        // CALL onFailAd
        return @"FAIL : Unable to fetch the top view controller";
    }
    
}

#pragma mark BIDMADBannerDelegate

- (void)BIDMADBannerLoad:(BIDMADBanner *)core {
    NSLog(@"%s", __FUNCTION__);
//    [self passToListenerWith: [NSString stringWithFormat:@"%s", __FUNCTION__]];
}

- (void)BIDMADBannerShow:(BIDMADBanner *)core {
    NSLog(@"%s", __FUNCTION__);
//    [self passToListenerWith: [NSString stringWithFormat:@"%s", __FUNCTION__]];
}

- (void)BIDMADBannerClick:(BIDMADBanner *)core {
    NSLog(@"%s", __FUNCTION__);
//    [self passToListenerWith: [NSString stringWithFormat:@"%s", __FUNCTION__]];
}

- (void)BIDMADBannerClosed:(BIDMADBanner *)core {
    NSLog(@"%s", __FUNCTION__);
//    [self passToListenerWith: [NSString stringWithFormat:@"%s", __FUNCTION__]];
}

- (void)BIDMADBannerError:(BIDMADBanner *)core code:(NSString *)error {
    NSLog(@"%s : %@", __FUNCTION__, error);
//    [self passToListenerWith: [NSString stringWithFormat:@"%s", __FUNCTION__]];
}

- (void)BIDMADBannerAllFail:(BIDMADBanner *)core {
    NSLog(@"%s", __FUNCTION__);
//    [self passToListenerWith: [NSString stringWithFormat:@"%s", __FUNCTION__]];
}

- (void)BIDMADBannerIntervalClosed:(BIDMADBanner *)core {
    NSLog(@"%s", __FUNCTION__);
    for (int i=0; i<self.bannerInstances.count; i++) {
        if (self.bannerInstances[i] == core) {
            [self.bannerInstances removeObjectAtIndex:i];
        }
    }
//    [self passToListenerWith: [NSString stringWithFormat:@"%s", __FUNCTION__]];
}

#pragma mark BIDMADInterstitialDelegate

-(void)BIDMADInterstitialLoad:(BIDMADInterstitial *)core {
    NSLog(@"%s", __FUNCTION__);
    
    [self setIsLoaded:YES];
    [[BidmadsdkFlutterPlugin.sharedInstance flutterChannel] invokeMethod:@"onLoadAd" arguments:core.zoneID];
//    [self passToListenerWith: [NSString stringWithFormat:@"%s", __FUNCTION__]];
//    [_interstitial showInterstitialView];
}

- (void)BIDMADInterstitialShow:(BIDMADInterstitial *)core {
    NSLog(@"%s", __FUNCTION__);
    [[BidmadsdkFlutterPlugin.sharedInstance flutterChannel] invokeMethod:@"onShowAd" arguments:core.zoneID];
}

- (void)BIDMADInterstitialClose:(BIDMADInterstitial *)core {
    NSLog(@"%s", __FUNCTION__);
//    [self passToListenerWith: [NSString stringWithFormat:@"%s", __FUNCTION__]];
    [[BidmadsdkFlutterPlugin.sharedInstance flutterChannel] invokeMethod:@"onCloseAd" arguments:core.zoneID];
    for (int i=0; i<self.interstitialInstances.count; i++) {
        if (self.interstitialInstances[i] == core) {
            [self.interstitialInstances removeObjectAtIndex:i];
        }
    }
}

- (void)BIDMADInterstitialError:(BIDMADInterstitial *)core code:(NSString *)error {
    NSLog(@"%s : %@", __FUNCTION__, error);
//    [self passToListenerWith: [NSString stringWithFormat:@"%s", __FUNCTION__]];
    [[BidmadsdkFlutterPlugin.sharedInstance flutterChannel] invokeMethod:@"onFailAd" arguments:core.zoneID];
}

- (void)BIDMADInterstitialAllFail:(BIDMADInterstitial *)core {
    NSLog(@"%s", __FUNCTION__);
//    [self passToListenerWith: [NSString stringWithFormat:@"%s", __FUNCTION__]];
    [[BidmadsdkFlutterPlugin.sharedInstance flutterChannel] invokeMethod:@"onFailAd" arguments:core.zoneID];
}

#pragma mark BIDMADRewardVideoDelegate

- (void)BIDMADRewardVideoLoad:(BIDMADRewardVideo *)core {
    NSLog(@"%s", __FUNCTION__);
    [self setIsLoaded:YES];
//    [self passToListenerWith: [NSString stringWithFormat:@"%s", __FUNCTION__]];
}

- (void)BIDMADRewardVideoShow:(BIDMADRewardVideo *)core {
    NSLog(@"%s", __FUNCTION__);
//    [self passToListenerWith: [NSString stringWithFormat:@"%s", __FUNCTION__]];
}

- (void)BIDMADRewardVideoClick:(BIDMADRewardVideo *)core {
    NSLog(@"%s", __FUNCTION__);
//    [self passToListenerWith: [NSString stringWithFormat:@"%s", __FUNCTION__]];
}

- (void)BIDMADRewardVideoClose:(BIDMADRewardVideo *)core {
    NSLog(@"%s", __FUNCTION__);
//    [self passToListenerWith: [NSString stringWithFormat:@"%s", __FUNCTION__]];
    for (int i=0; i<self.rewardVideoInstances.count; i++) {
        if (self.rewardVideoInstances[i] == core) {
            [self.rewardVideoInstances removeObjectAtIndex:i];
        }
    }
}

- (void)BIDMADRewardVideoSucceed:(BIDMADRewardVideo *)core {
    NSLog(@"%s", __FUNCTION__);
//    [self passToListenerWith: [NSString stringWithFormat:@"%s", __FUNCTION__]];
}

- (void)BIDMADRewardVideoComplete:(BIDMADRewardVideo *)core {
    NSLog(@"%s", __FUNCTION__);
//    [self passToListenerWith: [NSString stringWithFormat:@"%s", __FUNCTION__]];
}

- (void)BIDMADRewardVideoAllFail:(BIDMADRewardVideo *)core {
    NSLog(@"%s", __FUNCTION__);
//    [self passToListenerWith: [NSString stringWithFormat:@"%s", __FUNCTION__]];
}

- (void)BIDMADRewardSkipped:(BIDMADRewardVideo *)core {
    NSLog(@"%s", __FUNCTION__);
//    [self passToListenerWith: [NSString stringWithFormat:@"%s", __FUNCTION__]];
}

@end
